local abacus = AceLibrary("Abacus-2.0")
local L = AceLibrary("AceLocale-2.2"):new("Automaton")
local options = {
	type='group',
	args = {
		gossip = {
			type = 'toggle',
			name = 'AutoGossip',
			desc = 'Toggle AutoGossip on/off: Automatically complete quests and skip gossip text',
			get = function()
				return Automaton:IsEventRegistered("GOSSIP_SHOW")
			end,
			set = function(v)
				if v then
					Automaton.db.profile.autogossip = true
					Automaton:RegisterEvent("GOSSIP_SHOW")
					Automaton:RegisterEvent("QUEST_PROGRESS")
					Automaton:RegisterEvent("QUEST_COMPLETE")
					Automaton:RegisterEvent("QUEST_DETAIL")
				else
					Automaton.db.profile.autogossip = false
					Automaton:UnregisterEvent("GOSSIP_SHOW")
					Automaton:UnregisterEvent("QUEST_PROGRESS")
					Automaton:UnregisterEvent("QUEST_COMPLETE")
					Automaton:UnregisterEvent("QUEST_DETAIL")
				end
			end,
		},
		repair = {
			type = 'toggle',
			name = 'AutoRepair',
			desc = 'Toggle AutoRepair on/off: Automatically repair all inventory items when at merchant',
			get = function()
				return Automaton:IsEventRegistered("MERCHANT_SHOW")
			end,
			set = function(v)
				if v then
					Automaton.db.profile.autorepair = true
					Automaton:RegisterEvent("MERCHANT_SHOW")
				else
					Automaton.db.profile.autorepair = false
					Automaton:UnregisterEvent("MERCHANT_SHOW")
				end
			end,
		},
		trade = {
			type = 'toggle',
			name = 'AutoTrade',
			desc = 'Toggle AutoTrade on/off: Accept conjured items in trade window automatically',
			get = function()
				return Automaton:IsEventRegistered("TRADE_TARGET_ITEM_CHANGED")
			end,
			set = function(v)
				if v then
					Automaton.db.profile.autotrade = true
					Automaton:RegisterEvent("TRADE_TARGET_ITEM_CHANGED")
				else
					Automaton.db.profile.autotrade = false
					Automaton:UnregisterEvent("TRADE_TARGET_ITEM_CHANGED")
				end
			end,
		},
		rez = {
			type = 'toggle',
			name = 'AutoRez',
			desc = 'Toggle AutoRez on/off: Automatically accept resurrection requests. Use /rez instead if you want to manually specify each time to autorez.',
			get = function()
				return Automaton.db.profile.autorez
			end,
			set = function(v)
				if v then
					Automaton.db.profile.autorez = true
					Automaton:RegisterEvent("RESURRECT_REQUEST")
				else
					Automaton.db.profile.autorez = false
					Automaton:UnregisterEvent("RESURRECT_REQUEST")
				end
			end,
		},
		joinparty = {
			type = 'toggle',
			name = 'AutoJoinParty',
			desc = 'Toggle AutoJoinParty on/off: Accepts party invites from your friends or guild members automatically.',
			get = function()
				return Automaton:IsEventRegistered("PARTY_INVITE_REQUEST")
			end,
			set = function(v)
				if v then
					Automaton.db.profile.autojoingroup = true
					Automaton:RegisterEvent("PARTY_INVITE_REQUEST")
				else
					Automaton.db.profile.autojoingroup = false
					Automaton:UnregisterEvent("PARTY_INVITE_REQUEST")
				end
			end,
		},
	},
}

Automaton = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0","AceEvent-2.0","AceDebug-2.0","AceDB-2.0")
Automaton:RegisterChatCommand({"/auto", "/automaton"}, options)

Automaton:RegisterChatCommand({"/rez"}, {
	type = 'execute',
	desc = 'Manually set next rez to Auto Accept',
	func = function() Automaton:ManualAutoRez() end,
})

Automaton:RegisterDB("AutomatonDB", "AutomatonDBPC")
Automaton:RegisterDefaults("profile", {
    autorepair = true,
    autogossip = true,
	autotrade  = true,
	autorez    = false,
	autojoinparty = true,
} )

function Automaton:OnInitialize()
    -- Called when the addon is loaded
end

function Automaton:OnEnable()
	--AutoGossips
	if self.db.profile.autogossip then
		self:RegisterEvent("GOSSIP_SHOW")
		self:RegisterEvent("QUEST_PROGRESS")
		self:RegisterEvent("QUEST_COMPLETE")
		self:RegisterEvent("QUEST_DETAIL")
	end

	--AutoRepair
	if self.db.profile.autorepair then
		self:RegisterEvent("MERCHANT_SHOW")
	end
	
	--AutoTrade
	if self.db.profile.autotrade then
		self:RegisterEvent("TRADE_TARGET_ITEM_CHANGED")
	end
	
	--AutoRez
	if self.db.profile.autorez then
		self:RegisterEvent("RESURRECT_REQUEST")
	end
	
	if self.db.profile.autojoinparty then
		self:RegisterEvent("PARTY_INVITE_REQUEST")
	end
end

function Automaton:OnDisable()
	--AutoGossip
	self:UnregisterEvent("GOSSIP_SHOW")
	self:UnregisterEvent("QUEST_PROGRESS")
	self:UnregisterEvent("QUEST_COMPLETE")
	self:UnregisterEvent("QUEST_DETAIL")

	--AutoRepair
	self:UnregisterEvent("MERCHANT_SHOW")
	
	--AutoTrade
	self:UnregisterEvent("TRADE_TARGET_ITEM_CHANGED")
	
	--AutoRez
	self:UnregisterEvent("RESURRECT_REQUEST")

	--AutoJoinParty
	self:UnregisterEvent("PARTY_INVITE_REQUEST")
end

--AutoGossip
function Automaton:GOSSIP_SHOW()
	if IsShiftKeyDown() then return end

	local g = self:ProcessGossip(GetGossipOptions())

	if table.getn(g) > 1 then
		self:Debug("Too many gossips to pick from, doing nothing.")
		return
	elseif table.getn(g) == 1 then
		local z,_ = GetGossipAvailableQuests()
		local x,_ = GetGossipActiveQuests()
		if (x or z) and not (g[1][2] == "gossip") then
			self:Debug("Not Automatoning because there's an available or active quest.")
		else
			self:Debug(g[1][1])
			SelectGossipOption(g[1][3])
			return
		end
	end

	if self:CheckQuests(self:ProcessQuests(GetGossipActiveQuests()), SelectGossipActiveQuest) then
		return
	end
	if self:CheckQuests(self:ProcessQuests(GetGossipAvailableQuests()), SelectGossipAvailableQuest) then
		return
	end
end

function Automaton:QUEST_DETAIL()
	local q = GetTitleText()
	if AUTOMATON.AUTO_QUESTS[q] then
		self:Debug("AutoAccepting "..q..".")
		AcceptQuest()
	end
end

function Automaton:QUEST_PROGRESS()
	local q = GetTitleText()
	if AUTOMATON.AUTO_QUESTS[q] and IsQuestCompletable() then
		self:Debug("AutoCompleting "..q..".")
		CompleteQuest()
	end
end

function Automaton:QUEST_COMPLETE()
	local q = GetTitleText()
	if AUTOMATON.AUTO_QUESTS[q] then
		self:Debug("AutoRewardPicking "..q..".")
		GetQuestReward(0)
	end
end

----------------------
-- Helper Functions --
----------------------
function Automaton:SearchBagsForQuantity(itemname)
	local quantity = 0
	for bag = 0, 4 do
		if GetContainerNumSlots(bag) > 0 then
			for slot = 0, GetContainerNumSlots(bag) do
				if GetContainerItemLink(bag, slot) then
					local _,_,link = string.find(GetContainerItemLink(bag, slot), "(item:%d+:%d+:%d+:%d+)")
					local item = GetItemInfo(link)
				
					if item == itemname then
						local _,q = GetContainerItemInfo(bag, slot)
						quantity = quantity + q
					end
				end
			end
		end
	end
	return quantity
end

function Automaton:ProcessGossip(...)
	local gossips = {}
	for i = 1, table.getn(arg), 2 do
		local title, type = arg[i], arg[i+1]
		if AUTOMATON.AUTO_GOSSIP[type] then
			if table.getn(AUTOMATON.AUTO_GOSSIP[type]) == 0 then
				tinsert(gossips, {title, type, (i+1)/2})
			else
				for k,v in AUTOMATON.AUTO_GOSSIP[type] do
					if v == title then
						tinsert(gossips, {title, type, (i+1)/2})
					end
				end
			end
		end
	end
	return gossips
end

function Automaton:ProcessQuests(...)
	local quests = {}
	for i = 1, table.getn(arg), 2 do
		local title, level = arg[i], arg[i+1]
		if AUTOMATON.AUTO_QUESTS[title] then
			local good = true
			if AUTOMATON.AUTO_QUESTS[title].items then
				for k, v in AUTOMATON.AUTO_QUESTS[title].items do
					if tonumber(self:SearchBagsForQuantity(k)) < v then
						good = false
						break
					end
				end
			end
			if good then
				tinsert(quests, {title, level, (i+1)/2})
			end
		end
	end
	return quests
end

function Automaton:CheckQuests(quests, func)
	if table.getn(quests) > 1 then
		local quest, priority = nil, 0
		for k,v in quests do
			self:Debug(k,v)
			if AUTOMATON.AUTO_QUESTS[v[1]].priority and AUTOMATON.AUTO_QUESTS[v[1]].priority > priority then
				priority = AUTOMATON.AUTO_QUESTS[v[1]].priority
				quest = k
			end
		end
		self:Debug("AutoActiveQuest: "..quests[quest][1])
		func(quests[quest][3])
		return true
	elseif table.getn(quests) == 1 then
		self:Debug("AutoActiveQuest: "..quests[1][1])
		func(quests[1][3])
		return true
	end
	return false
end

-- AutoRepair

function Automaton:MERCHANT_SHOW()
	if CanMerchantRepair() then
		local repairCost = GetRepairAllCost()
		if repairCost > 0 then
			self:Print("Repairing all items for: %s", abacus:FormatMoneyFull(repairCost, true))
			RepairAllItems()
		end
	end
end

-- AutoTrade (Will die with WoW v2.0)
function Automaton:TRADE_TARGET_ITEM_CHANGED()
	if self:IsEventScheduled("Automaton_autotrade") then
		self:CancelScheduledEvent("Automaton_autotrade")
	end
	
	if GetPlayerTradeMoney() > 0 or GetTargetTradeMoney() > 0 then
		self:Debug("Oops. You or the target to trade with have money in the window.")
		return
	end
	local items = 0
	for i=1,7 do
		local name = GetTradeTargetItemInfo(i)
		if name and not foreachi(AUTOMATON.AUTO_TRADE["Accept"],function(i,v) if v==name then return true end end) then
			self:Debug("Oops. I'm not allowed to take one of those items.")
			return
		elseif name then
			items = items + 1
		end
		name = GetTradePlayerItemInfo(i)
		if name then
			self:Debug("Oops. Item in my trade window, no dice.")
			return
		end
	end
	if items == 0 then return end
	
	local slots = 0
	for i=0,4 do
		for j=1,GetContainerNumSlots(i) do
			if not GetContainerItemInfo(i,j) then
				slots = slots + 1
			end
		end
	end

	if slots < items then
		self:print("Oops. I don't have the space in my inventory for this.")
		return
	end

	self:Debug("AutoTrading...")
	self:ScheduleEvent("Automaton_autotrade", function() AcceptTrade() end, 1, self)
end


-- AutoRez

function Automaton:ManualAutoRez()
	if not self.db.profile.autorez then
		if self:IsEventRegistered("RESURRECT_REQUEST") then
			self:UnregisterEvent("RESURRECT_REQUEST")
			self:Print("AutoResurrect cancelled.")
		else
			self:RegisterEvent("RESURRECT_REQUEST")
			self:Print("Automatically accepting next rez.")
		end
	else
		self:Print("AutoRez is already automatically accepting all resurrection requests. If you prefer to manually specify each time to auto-accept, use /automaton rez to disable permenant AutoRez and try again.")
	end
end

function Automaton:RESURRECT_REQUEST()
	AcceptResurrect()
	StaticPopup_Hide("RESURRECT_NO_SICKNESS")
	if not self.db.profile.autorez then
		self:UnregisterEvent("RESURRECT_REQUEST")
	end
end

-- Auto Join Party
function Automaton:PARTY_INVITE_REQUEST()
	self:Debug("Processing AutoJoinParty")
	local who = arg1
	local acceptFrom = {}
	
	for i=1,GetNumGuildMembers() do
		local name = GetGuildRosterInfo(i)
		tinsert(acceptFrom,name)
	end
	
	for i=1,GetNumFriends() do
		local name = GetFriendInfo(i)
		tinsert(acceptFrom,name)
	end
	
	if foreachi(acceptFrom, function(i,v) if v==who then return true end end) then
		self:Debug("AutoAccepting an invite from ".. who)
		AcceptGroup()
		StaticPopupDialogs["PARTY_INVITE"].inviteAccepted = 1
		StaticPopup_Hide("PARTY_INVITE")
	end
end

-- Auto BG Release


-- Auto BG Join

